/*

  This is a part of LSDynamic.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <math.h>
#include "LSDynamic.h"

CCaption::CCaption()
{
	hWnd = NULL;
	hwnd = string("ShortcutClass");
	alpha.enable = false;
	cornerRelative = false;
}

/************************************************************************************/

CCaption::CCaption(const char *n)
{
	hWnd = NULL;
	name = string(n);
	hwnd = string("ShortcutClass");
	alpha.enable = false;
	cornerRelative = false;
}

/************************************************************************************/

CCaption::CCaption(const class CCaption &cap)
{
	hWnd	= cap.hWnd;
	hwnd	= cap.hwnd;
	name	= cap.name;

	size	= cap.size;
	pos		= cap.pos;
	center	= cap.center;

	angle	= cap.angle;
	ray		= cap.ray;
	alpha	= cap.alpha;

	move	= cap.move;
	internalAlpha = cap.internalAlpha;
	cornerRelative = cap.cornerRelative;
}

/************************************************************************************/

CCaption& CCaption::operator=(const CCaption& cap)
{
	hWnd	= cap.hWnd;
	hwnd	= cap.hwnd;
	name	= cap.name;

	size	= cap.size;
	pos		= cap.pos;
	center	= cap.center;

	angle	= cap.angle;
	ray		= cap.ray;
	alpha	= cap.alpha;

	move	= cap.move;
	internalAlpha = cap.internalAlpha;
	cornerRelative = cap.cornerRelative;

	return(*this);
}

/************************************************************************************/

int CCaption::initAngle(t_pos &p)
{
	float ret, ang;
	int dx, dy;

	dx = p.x - center.initial.x;
	dy = p.y - center.initial.y;
	if (dx)
	{
		ret = ((dx > 0) ? dx : (-dx)) / (float)ray.initial;
		if (ret != 0)
			ang = atan(sqrt(1 - pow(ret, 2)) / ret);
		else
			ang = M_PI / 2;

		if (dx < 0) ang = -ang + M_PI;
		if (dy < 0) ang = -ang;
	}
	else
		ang = (dy > 0) ? (M_PI / 2) : (-M_PI / 2);

	return (ang * 180.0f / M_PI);
}

/************************************************************************************/

void CCaption::getPos(t_range_xy &range, float d_delta)
{
	range.current.x = range.initial.x + range.delta.x * d_delta;
	range.current.y = range.initial.y + range.delta.y * d_delta;
}

/************************************************************************************/

void CCaption::update(const CDyn *dyn, bool update_pos)
{
	if (!hWnd)
		return;

	if (dyn->info.cur_time >= dyn->info.end_time)
	{
		/*// not very useful...
		if ((alpha.final == 255) && DynStorage->restoreLayer)
			restoreLayer();
			/**/
		alpha.current = alpha.final;
		size.current = size.final;

		if (move == dyn_linear)
			pos.current = pos.final;
		else
		{
			pos.current = center.final;
			pos.current.x += ray.final * cos(angle.final * M_PI / 180.0f);
			pos.current.y += ray.final * sin(angle.final * M_PI / 180.0f);
		}
	}
	else
	{
		if (alpha.enable)
				alpha.current = alpha.initial + alpha.delta * dyn->alpha.delta;

		if (size.enable)
			getPos(size, dyn->cur_delta_time);

		if (move == dyn_linear)
			getPos(pos, dyn->linear_acc);
		else
		{
			if (ray.enable)
				ray.current = ray.initial + ray.delta * dyn->ray_acc;

			if (center.enable)
				getPos(center, dyn->center_acc);

			angle.current = angle.initial + angle.delta * dyn->angle_acc;

			pos.current = center.current;
			pos.current.x += ray.current * cos(angle.current * M_PI / 180.0f);
			pos.current.y += ray.current * sin(angle.current * M_PI / 180.0f);
		}
	}
	if (update_pos)
		setPos();
}

/************************************************************************************/

bool CCaption::init(const CDyn *dyn)
{
	WINDOWPLACEMENT win_pos;
	char buffer[MAX_LINE_LENGTH];

	if (dyn->info.has_hwnd)
		hwnd = dyn->info.hwnd;

	if (!hWnd)
		DynStorage->getWindow(this);

	if (!hWnd)
		return true;
	move = dyn->info.type;

	GetWindowPlacement(hWnd, &win_pos);
	if (dyn->pos.enable)
	{
		if (dyn->pos.x_relative)
			pos.initial.x = win_pos.rcNormalPosition.left + dyn->pos.x;
		else
			pos.initial.x = dyn->pos.x;

		if (dyn->pos.y_relative)
			pos.initial.y = win_pos.rcNormalPosition.top + dyn->pos.y;
		else
			pos.initial.y = dyn->pos.y;
	}
	else
	{
		pos.initial.x = win_pos.rcNormalPosition.left;
		pos.initial.y = win_pos.rcNormalPosition.top;
	}

	size.initial.x = win_pos.rcNormalPosition.right - win_pos.rcNormalPosition.left;
	size.initial.y = win_pos.rcNormalPosition.bottom - win_pos.rcNormalPosition.top;
	if (dyn->size.enable)
	{
		size.enable = true;
		size.final.x = dyn->size.x;
		size.final.y = dyn->size.y;
		size.current.x = size.final.x - size.initial.x;
		size.current.y = size.final.y - size.initial.y;
		size.delta.x = (float)size.current.x / dyn->delta_time;
		size.delta.y = (float)size.current.y / dyn->delta_time;
	}
	else
	{
		size.enable = false;
		size.final = size.initial;
	}
	size.current = size.initial;

	cornerRelative = DynStorage->relativeWindowCenter ? !dyn->cornerRelative : dyn->cornerRelative;
	if ((move == dyn_circular) && cornerRelative)
	{
		pos.initial.x += size.initial.x / 2;
		pos.initial.y += size.initial.y / 2;
	}

	// linear position

	if (dyn->dest.enable)
	{
		if (dyn->dest.x_relative)
			pos.final.x = pos.initial.x + dyn->dest.x;
		else
			pos.final.x = dyn->dest.x;

		if (dyn->dest.y_relative)
			pos.final.y = pos.initial.y + dyn->dest.y;
		else
			pos.final.y = dyn->dest.y;
	}
	else
		pos.final = pos.initial;

	pos.current.x = pos.final.x - pos.initial.x;
	pos.current.y = pos.final.y - pos.initial.y;
	pos.delta.x = (float)pos.current.x / dyn->delta_time;
	pos.delta.y = (float)pos.current.y / dyn->delta_time;


	// circular position

	if (dyn->center.enable)
	{
		if (dyn->center.x_relative)
			center.initial.x = pos.initial.x + dyn->center.x + (dyn->center.x ? 0 : 1);
		else
			center.initial.x = dyn->center.x;

		if (dyn->center.y_relative)
			center.initial.y = pos.initial.y + dyn->center.y + (dyn->center.y ? 0 : 1);
		else
			center.initial.y = dyn->center.y;
	}
	else
		center.initial = pos.initial;


	if (dyn->c_dest.enable)
	{
		center.enable = true;
		if (dyn->c_dest.x_relative)
			center.final.x = center.initial.x + dyn->c_dest.x;
		else
			center.final.x = dyn->c_dest.x;

		if (dyn->c_dest.y_relative)
			center.final.y = center.initial.y + dyn->c_dest.y;
		else
			center.final.y = dyn->c_dest.y;
	}
	else
	{
		center.enable = false;
		center.final = center.initial;
	}


	center.current.x = center.final.x - center.initial.x;
	center.current.y = center.final.y - center.initial.y;
	center.delta.x = (float)center.current.x / dyn->delta_time;
	center.delta.y = (float)center.current.y / dyn->delta_time;

	if ((center.initial.x == pos.initial.x) && (center.initial.y == pos.initial.y))
		ray.initial = 0;
	else
		ray.initial = sqrt(pow(center.initial.x - pos.initial.x, 2) + pow(center.initial.y - pos.initial.y, 2));
	ray.current = ray.initial;
	if (dyn->r_dest == -1)
		ray.final = ray.initial;
	else
	{
		ray.final = dyn->r_dest;
		ray.enable = true;
	}
	ray.delta = (ray.final - ray.initial) / dyn->delta_time;

	angle.initial = initAngle(pos.initial);
	angle.current = angle.initial;
	angle.final = angle.initial + dyn->angle;
	angle.delta = (angle.final - angle.initial) / dyn->delta_time;

	internalAlpha = (DynStorage->b_layeredShortcut && (hwnd.compare("ShortcutClass") == 0)) || DynStorage->hasInternalAlpha(hwnd);
	alpha = dyn->alpha;
	//alpha.enable = false;
	if (alpha.enable)
	{
		alpha.delta = (alpha.final - alpha.initial) / dyn->delta_time;
		alpha.current = alpha.initial;
		SetWindowLong(hWnd, GWL_EXSTYLE,GetWindowLong(hWnd, GWL_EXSTYLE) | WS_EX_LAYERED);
		updateAlpha();
	}

	pos.current = pos.initial;
	center.current = center.initial;

	return false;
}

/************************************************************************************/

void CCaption::updateAlpha()
{
	if (internalAlpha)
		::SendMessage(hWnd, WM_ALPHA, 0, alpha.current);
	else
		DynStorage->SetLayeredWindowAttributes(hWnd, 0, alpha.current, LWA_ALPHA);
}

/************************************************************************************/

void CCaption::restore()
{
	pos.current = pos.initial;
	setPos();
	if (DynStorage->restoreLayer)
		restoreLayer();
}

/************************************************************************************/

void CCaption::restoreLayer()
{
	if (alpha.enable && hWnd)// && (alpha.final == 255) )//&& (hwnd.compare("ShortcutClass") == 0))
	{
		if (internalAlpha)
			::SendMessage(hWnd, WM_ALPHA, 0, -1);
		else
		{
			alpha.enable = false;
			if (GetWindowLong(hWnd, GWL_EXSTYLE) & WS_EX_LAYERED)
			{
				SetWindowLong(hWnd, GWL_EXSTYLE,GetWindowLong(hWnd, GWL_EXSTYLE) & ~WS_EX_LAYERED);
				RedrawWindow(hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_FRAME | RDW_ALLCHILDREN);
			}
		}
	}
}

/************************************************************************************/

void CCaption::setPos()
{
	if ((move == dyn_circular) && cornerRelative)
		SetWindowPos(hWnd, NULL, pos.current.x - size.current.x / 2, pos.current.y - size.current.y / 2, size.current.x, size.current.y, SWP_NOZORDER|SWP_NOACTIVATE);
	else
		SetWindowPos(hWnd, NULL, pos.current.x, pos.current.y, size.current.x, size.current.y, SWP_NOZORDER|SWP_NOACTIVATE);/**/
	/*if ((move == dyn_circular) && DynStorage->relativeWindowCenter)
		MoveWindow(hWnd, pos.current.x - size.x / 2, pos.current.y - size.y / 2, size.x, size.y, true);
	else
		MoveWindow(hWnd, pos.current.x, pos.current.y, size.x, size.y, true);/**/
	
	if (alpha.enable)
		updateAlpha();
}

/************************************************************************************/

CCaption::~CCaption()
{
	//restoreLayer();
}