/*

  This is a part of LSDynamic.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "LSDynamic.h"

CDyn::CDyn(e_dyn e, FILE *f, char *n):
angle(0), r_dest(-1), acc(100), r_acc(100), c_acc(100), a_acc(100)
{
	cornerRelative = false;
	alpha.current = 100;
	alpha.enable = false;
	alpha.initial = alpha.final = 255;
	pos.enable = false;
	dest.enable = false;
	center.enable = false;
	c_dest.enable = false;
	size.enable = false;
	info.type = e;
	load(f, n);
}

/************************************************************************************/

CDyn::CDyn(const class CDyn &dyn)
{
	info		= dyn.info;

	size		= dyn.size;
	pos			= dyn.pos;
	dest		= dyn.dest;
	center		= dyn.center;
	c_dest		= dyn.c_dest;
	r_dest		= dyn.r_dest;
	angle		= dyn.angle;
	acc			= dyn.acc;
	r_acc		= dyn.r_acc;
	c_acc		= dyn.c_acc;
	a_acc		= dyn.a_acc;
	alpha		= dyn.alpha;

	delta_time	= dyn.delta_time;
	cornerRelative = dyn.cornerRelative;
}

/************************************************************************************/

CDyn& CDyn::operator=(const CDyn& dyn)
{
	info		= dyn.info;

	size		= dyn.size;
	pos			= dyn.pos;
	dest		= dyn.dest;
	center		= dyn.center;
	c_dest		= dyn.c_dest;
	r_dest		= dyn.r_dest;
	angle		= dyn.angle;
	acc			= dyn.acc;
	r_acc		= dyn.r_acc;
	c_acc		= dyn.c_acc;
	a_acc		= dyn.a_acc;
	alpha		= dyn.alpha;

	delta_time	= dyn.delta_time;
	cornerRelative = dyn.cornerRelative;

	return(*this);
}

/************************************************************************************/

void CDyn::getPos(t_pos *p, char *token1, char *token2, bool relative)
{
	p->x = atoi(token1);
	p->y = atoi(token2);
	if (relative)
	{
		p->x_relative = (*token1 == '+') || (*token1 == '-');
		p->y_relative = (*token2 == '+') || (*token2 == '-');
	}
	else
	{
		p->x_relative = false;
		p->y_relative = false;
		p->x = CONVERT_COORDINATE_X(p->x);
		p->y = CONVERT_COORDINATE_Y(p->y);
	}
}

/************************************************************************************/

void CDyn::load(FILE *f, char *n)
{
	char configLine[MAX_LINE_LENGTH],
		 token1[MAX_LINE_LENGTH],
		 token2[MAX_LINE_LENGTH],
		 extra[MAX_LINE_LENGTH];
	char *tokens[] = {token1, token2};

	info.name = string(n);
	while(LCReadNextConfig(f, "*dyn", configLine, MAX_LINE_LENGTH))
	{
		(void) LCTokenize (configLine, tokens, 2, extra);
		if (strcmp(token2, (info.type == dyn_linear) ? "~linear" : "~circular") == 0)
			break;

		if (stricmp(token2, "hwnd") == 0)
			info.getHWND(trimQuotes(extra));
		else if (stricmp(token2, "caption") == 0)
			info.getCaption(extra);
		else if (stricmp(token2, "range") == 0)
		{
			LCTokenize (extra, tokens, 1, configLine);
			info.start_time = atoi(token1);
			info.end_time = atoi(configLine);
		}
		else if (stricmp(token2, "alpha") == 0)
		{
			LCTokenize (extra, tokens, 1, configLine);
			alpha.initial = atoi(token1);
			alpha.final = atoi(configLine);
			alpha.enable = true;
		}
		else if (stricmp(token2, "size") == 0)
		{
			LCTokenize (extra, tokens, 2, configLine);
			getPos(&size, token1, token2, StrLen(configLine) != 0);
			size.enable = true;
		}
		else if (stricmp(token2, "pos") == 0)
		{
			LCTokenize (extra, tokens, 2, configLine);
			getPos(&pos, token1, token2, StrLen(configLine) != 0);
			pos.enable = true;
		}
		else if (stricmp(token2, "dest") == 0)
		{
			LCTokenize (extra, tokens, 2, configLine);
			getPos(&dest, token1, token2, StrLen(configLine) != 0);
			dest.enable = true;
		}
		else if (stricmp(token2, "center") == 0)
		{
			LCTokenize (extra, tokens, 2, configLine);
			getPos(&center, token1, token2, StrLen(configLine) != 0);
			center.enable = true;
		}
		else if (stricmp(token2, "c-dest") == 0)
		{
			LCTokenize (extra, tokens, 2, configLine);
			getPos(&c_dest, token1, token2, StrLen(configLine) != 0);
			c_dest.enable = true;
		}
		else if (stricmp(token2, "angle") == 0)
			angle = atoi(extra);
		else if (stricmp(token2, "flags") == 0)
			cornerRelative = strchr(extra, 'c') || strchr(extra, 'C');
		else if (stricmp(token2, "r-dest") == 0)
			r_dest = atoi(extra);
		else if (stricmp(token2, "acc") == 0)
			acc = atoi(extra);
		else if (stricmp(token2, "r-acc") == 0)
			r_acc = atoi(extra);
		else if (stricmp(token2, "c-acc") == 0)
			c_acc = atoi(extra);
		else if (stricmp(token2, "a-acc") == 0)
			a_acc = atoi(extra);
		else if (stricmp(token2, "alpha-acc") == 0)
			alpha.current = atoi(extra);
	}
}

/************************************************************************************/

void CDyn::init(CDynMacro *owner, CDynType *window)
{
	int i;

	if (!info.has_hwnd)
	{
		if (window->has_hwnd)
			info.getHWND(window->hwnd.c_str());
		else if (owner->info.has_hwnd)
			info.getHWND(owner->info.hwnd.c_str());
		else
			info.getHWND(owner->args.hwnd.c_str());
	}

	if (!info.has_caption)
	{
		if (window->has_caption)
			info.copyCaption(window->caption);
		else if (owner->info.has_caption)
			info.copyCaption(owner->info.caption);
		else
			info.copyCaption(owner->args.caption);
	}

	if (!(info.start_time && info.end_time))
	{
		info.start_time = window->start_time;
		info.end_time = window->end_time;
	}
	else
	{
		info.start_time += window->start_time;
		info.end_time += window->start_time;
	}
	info.cur_time = info.start_time;
	delta_time = (info.end_time - info.start_time) / TimerUpdateDelay;

	alpha.enable = DynStorage->has_alpha;
	info.is_running = 0;
}

/************************************************************************************/

void CDyn::update(long t, bool update_pos, bool reverse)
{
	int i;
	float d_delta;
	CaptionListIterator it;

	info.cur_time = t;

	if (info.cur_time < info.start_time)
		return;

	if (info.is_running == 0)
	{
		info.is_running = 1;
		while (info.caption.front()->name.find('*') < info.caption.front()->name.size())
		//if (info.caption.front()->name.compare("*") == 0)
			DynStorage->getWindowStar(&info);
		it = info.caption.begin();
		while (it != info.caption.end())
		{
			if ((*it)->init(this))
			{
				delete (*it);
				it = info.caption.erase(it);
			}
			else
				it++;
		}
	}

	if (info.cur_time >= info.end_time)
	{
		info.cur_time = info.end_time;
		info.is_running = 2;
	}

	cur_delta_time	= (info.cur_time - info.start_time) / TimerUpdateDelay;
	d_delta			= (delta_time - cur_delta_time) / delta_time;
	if (acc != 100)
		linear_acc	= (cur_delta_time * (100.0f + d_delta * (100 - acc))) / 100.0f;
	else
		linear_acc = cur_delta_time;

	if (r_acc != 100)
		ray_acc		= (cur_delta_time * (100.0f + d_delta * (100 - r_acc))) / 100.0f;
	else
		ray_acc = cur_delta_time;

	if (c_acc != 100)
		center_acc	= (cur_delta_time * (100.0f + d_delta * (100 - c_acc))) / 100.0f;
	else
		center_acc = cur_delta_time;

	if (a_acc != 100)
		angle_acc	= (cur_delta_time * (100.0f + d_delta * (100 - a_acc))) / 100.0f;
	else
		angle_acc = cur_delta_time;

	if (alpha.current != 100)
		alpha.delta	= (cur_delta_time * (100.0f + d_delta * (100 - alpha.current))) / 100.0f;
	else
		alpha.delta = cur_delta_time;

	if (reverse)
	{
		list<CCaption *>::reverse_iterator rit;
		for (rit = info.caption.rbegin(); rit != info.caption.rend(); rit++)
			(*rit)->update(this, update_pos);
	}
	else
		for (it = info.caption.begin(); it != info.caption.end(); it++)
			(*it)->update(this, update_pos);
}

/************************************************************************************/

CDyn::~CDyn()
{
}
