/*

  This is a part of LSDynamic.dll source code.
  
  Copyright (C) 2001-2002 Sylvain Rouquette
  rouque_s@epita.fr
  http://www.epita.fr:8000/~rouque_s

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include "LSDynamic.h"

CDynMacro::CDynMacro(FILE *f, char *n):
b_loop(false), b_one_exec(false), b_restore(false), is_paused(false), b_reverse(false), timer(-1)
{
	load(f, n);
}

/************************************************************************************/

CDynMacro::CDynMacro(const class CDynMacro &dyn):
b_restore(false), is_paused(false)
{
	CDynType *dynType;
	int i;

	info		= dyn.info;
	args		= dyn.args;
	b_restore	= dyn.b_restore;
	b_relay		= dyn.b_relay;
	b_loop		= dyn.b_loop;
	b_one_exec	= dyn.b_one_exec;
	b_reverse	= dyn.b_reverse;
	restore		= dyn.restore;
	relay		= dyn.relay;
	relay_mode	= dyn.relay_mode;
	timer		= dyn.timer;

	if (!dyn.dynTypeVector.empty())
	{
		dynTypeVector.reserve(dyn.dynTypeVector.size());
		for (i = 0; i < dyn.dynTypeVector.size(); i++)
			dynTypeVector.push_back(dynType = new CDynType(*dyn.dynTypeVector[i]));
	}
}

/************************************************************************************/

CDynMacro& CDynMacro::operator=(const CDynMacro& dyn)
{
	CDynType *dynType;
	int i;

	info		= dyn.info;
	args		= dyn.args;
	b_restore	= dyn.b_restore;
	b_relay		= dyn.b_relay;
	b_loop		= dyn.b_loop;
	b_one_exec	= dyn.b_one_exec;
	b_reverse	= dyn.b_reverse;
	restore		= dyn.restore;
	relay		= dyn.relay;
	relay_mode	= dyn.relay_mode;
	timer		= dyn.timer;

	if (!dynTypeVector.empty())
	{
		dynTypeVector.reserve(dyn.dynTypeVector.size());
		for (i = 0; i < dyn.dynTypeVector.size(); i++)
			dynTypeVector.push_back(dynType = new CDynType(*dyn.dynTypeVector[i]));
	}

	return (*this);
}

/************************************************************************************/

e_dyn CDynMacro::retDynType(const char *str)
{
	if (stricmp(str, "move") == 0)
		return (dyn_move);
	else if (stricmp(str, "exec") == 0)
		return (dyn_command);
	else if (stricmp(str, "pause") == 0)
		return (dyn_pause);
	else if (stricmp(str, "resume") == 0)
		return (dyn_resume);
	else if (stricmp(str, "win-stats") == 0)
		return (dyn_win_stats);

	return(dyn_none);
}

/************************************************************************************/

void CDynMacro::load(FILE *f, char *n)
{
	int ret;
	CDynType *dynType;

	char configLine[MAX_LINE_LENGTH],
		 token1[MAX_LINE_LENGTH],
		 token2[MAX_LINE_LENGTH],
		 extra[MAX_LINE_LENGTH];
	char *tokens[] = {token1, token2};

	if (*n != '!')
	{
		info.name.append(1, '!');
		info.name += string(n);
		
	}
	else
		info.name = string(n);
	AddBangCommandEx(info.name.c_str(), DynBangCommand);
	while(LCReadNextConfig(f, "*dyn", configLine, MAX_LINE_LENGTH))
	{
		ret = LCTokenize (configLine, tokens, 2, extra);
		if (stricmp(token2, "~macro") == 0)
			break;

		if (stricmp(token2, "linear") == 0)
			DynStorage->addDyn(f, dyn_linear, trimQuotes(extra));
		else if (stricmp(token2, "circular") == 0)
			DynStorage->addDyn(f, dyn_circular, trimQuotes(extra));
		else if (stricmp(token2, "flags") == 0)
		{
			b_loop = (strchr(extra, 'L') != NULL) || (strchr(extra, 'l') != NULL);
			b_one_exec = (strchr(extra, 'O') != NULL) || (strchr(extra, 'o') != NULL);
			b_reverse = (strchr(extra, 'R') != NULL) || (strchr(extra, 'r') != NULL);
		}
		else if (stricmp(token2, "timer") == 0)
			timer = atoi(extra);
		else if (stricmp(token2, "relay") == 0)
		{
			LCTokenize (extra, tokens, 1, configLine);
			if (stricmp(token1, "sync") == 0)
				relay_mode = relay_sync;
			else if (stricmp(token1, "give-up") == 0)
				relay_mode = relay_giveup;
			else if (stricmp(token1, "speed-up") == 0)
				relay_mode = relay_speedup;
			else if (stricmp(token1, "default") == 0)
				relay_mode = relay_default;
			b_relay = getArgs(relay, configLine);
		}
		else if (stricmp(token2, "restore") == 0)
			b_restore = getArgs(restore, extra);
		else if (stricmp(token2, "hwnd") == 0)
			info.getHWND(trimQuotes(extra));
		else if (stricmp(token2, "caption") == 0)
			info.getCaption(extra);
		else
		{
			dynType = new CDynType;
			dynType->type = retDynType(trimQuotes(token2));
			if (dynType->type == dyn_none)
			{
				delete dynType;
				continue;
			}
			dynTypeVector.push_back(dynType);
			ret = LCTokenize (extra, tokens, 2, configLine);
			dynType->start_time = atol(token1);
			dynType->end_time   = atol(token2);
			if (info.end_time < dynType->end_time)
				info.end_time = dynType->end_time;
			if ((dynType->type == dyn_move) || (dynType->type == dyn_win_stats))
			{
				ret = LCTokenize (configLine, tokens, 2, extra);
				dynType->name = string(trimQuotes(token1));
				if (ret >= 2)
				{
					dynType->getHWND(trimQuotes(token2));
					dynType->getCaption(extra);
				}

			}
			else
				dynType->name = string(trimQuotes(configLine));
		}
	}
}

/************************************************************************************/

void CDynMacro::init(const char *arguments, int t)
{
	char token1[MAX_LINE_LENGTH],
		 extra[MAX_LINE_LENGTH];
	char *tokens[] = {token1};
	int i;

	if (timer < 0)
		timer = t;

	if (!StrLen(arguments))
	{
		for (i = 0; i < dynTypeVector.size(); i++)
			dynTypeVector[i]->hwnd = info.hwnd;
	}
	else
	{
		if (stricmp(arguments, "default") != 0)
		{
			LCTokenize (arguments, tokens, 1, extra);
			args.getHWND(token1);
			args.getCaption(extra);
		}
		else
		{
			/*for (i = 0; i < dynTypeVector.size(); i++)
				dynTypeVector[i]->hwnd = info.hwnd;*/
			args.getHWND(info.hwnd.c_str());
		}
	}
	info.is_running = false;
}

/************************************************************************************/

bool CDynMacro::update()
{
	return (updateMovement(timer, true));
}

/************************************************************************************/

bool CDynMacro::updateMovement(int t, bool update_pos)
{
	int i, j;

	if (!is_paused)
		info.cur_time += t;

	for (i = 0; i < dynVector.size(); i++)
		if (dynVector[i]->info.is_running < 2)
			dynVector[i]->update(info.cur_time, update_pos, b_reverse);

	for (i = dynTypeVector.size() - 1; i >= 0; i--)
	{
		if ((info.cur_time >= dynTypeVector[i]->start_time) &&
			(info.cur_time <= dynTypeVector[i]->end_time))
		{
			if (dynTypeVector[i]->type == dyn_command)
			{
				DynStorage->exec(dynTypeVector[i]->name.c_str());
				if (b_one_exec)
				{
					dynTypeVector.erase(&dynTypeVector[i]);
				}
			}
			else if (dynTypeVector[i]->type == dyn_pause)
				DynStorage->pause(dynTypeVector[i]->name);
			else if (dynTypeVector[i]->type == dyn_resume)
				DynStorage->resume(dynTypeVector[i]->name);
			else if (dynTypeVector[i]->type == dyn_win_stats)
			{
				while (dynTypeVector[i]->caption.front()->name.find('*') < dynTypeVector[i]->caption.front()->name.size())
					DynStorage->getWindowStar(dynTypeVector[i]);
				for (CaptionListIterator it = dynTypeVector[i]->caption.begin();
					 it != dynTypeVector[i]->caption.end(); it++)
				{
					if (!(*it)->hWnd)
					{
						(*it)->hwnd = dynTypeVector[i]->hwnd;
						DynStorage->getWindow((*it));
					}
					if ((*it)->hWnd)
						ShowWindow((*it)->hWnd, (dynTypeVector[i]->name.compare("hide") == 0) ? SW_HIDE : SW_SHOWNA);
				}
			}
		}

	}


	if (info.cur_time >= info.end_time)
		if (b_loop)
		{
			info.cur_time = 0;
			for (i = 0; i < dynVector.size(); i++)
				dynVector[i]->info.is_running = 0;
		}
		else
			info.is_running = true;

	return (info.is_running);
}

/************************************************************************************/

void CDynMacro::restoreCaption()
{
	int i, j, k;

	for (j = dynVector.size() - 1; j >= 0; j--)
		for (CaptionListIterator it = dynVector[j]->info.caption.begin();
			 it != dynVector[j]->info.caption.end(); it++)
			for (i = restore.size() - 1; i >= 0; i--)
				if (restore[i].compare((*it)->name) == 0)
				{
					(*it)->restore();
					restore.erase(&restore[i]);
					break;
				}
}

/************************************************************************************/

void CDynMacro::relayDyn()
{
}

/************************************************************************************/

CDynMacro::~CDynMacro()
{
	int i;

	if (b_restore)
		restoreCaption();

	if (b_relay)
		relayDyn();

	for (i = 0; i < dynVector.size(); i++)
		delete dynVector[i];
	dynVector.clear();

	for (i = 0; i < dynTypeVector.size(); i++)
		delete dynTypeVector[i];
	dynTypeVector.clear();
}
